@@ -30,6 +30,7 @@ from wapitiCore.definitions.sql import SqlInjectionFinding
 from wapitiCore.definitions.internal_error import InternalErrorFinding
 from wapitiCore.model import PayloadInfo
 from wapitiCore.net import Request, Response
+from wapitiCore.net.web import http_repr
 from wapitiCore.parsers.ini_payload_parser import IniPayloadReader, replace_tags
 
 
@@ -91,7 +92,6 @@ class ModuleTimesql(Attack):
                     log_message = Messages.MSG_PARAM_INJECT
 
                 await self.add_critical(
-                    request_id=request.path_id,
                     finding_class=SqlInjectionFinding,
                     request=mutated_request,
                     info=vuln_message,
@@ -106,7 +106,7 @@ class ModuleTimesql(Attack):
                     parameter.display_name
                 )
                 log_red(Messages.MSG_EVIL_REQUEST)
-                log_red(mutated_request.http_repr())
+                log_red(http_repr(mutated_request))
                 log_red("---")
 
                 # We reached maximum exploitation for this parameter, don't send more payloads
@@ -124,7 +124,6 @@ class ModuleTimesql(Attack):
                         anom_msg = Messages.MSG_PARAM_500.format(parameter.display_name)
 
                     await self.add_high(
-                        request_id=request.path_id,
                         finding_class=InternalErrorFinding,
                         request=mutated_request,
                         info=anom_msg,
@@ -135,5 +134,5 @@ class ModuleTimesql(Attack):
                     log_orange("---")
                     log_orange(Messages.MSG_500, page)
                     log_orange(Messages.MSG_EVIL_REQUEST)
-                    log_orange(mutated_request.http_repr())
+                    log_orange(http_repr(mutated_request))
                     log_orange("---")
