@@ -34,11 +34,11 @@ from wapiti_arsenic.errors import JavascriptError, UnknownError, ArsenicError
 from wapitiCore.attack.cve.checker import (
     CVEChecker, cvss_score_to_wapiti_level, CVE_DIRECTORY, SUPPORTED_SOFTWARES, is_cve_supported_software
 )
+from wapitiCore.controller.exceptions import InvalidOptionValue
 from wapitiCore.definitions.vulnerable_software_version import VulnerableSoftwareFinding
 from wapitiCore.main.log import logging, log_blue, log_severity
-from wapitiCore.main.wapiti import is_valid_url
+from wapitiCore.net.web import is_valid_url
 from wapitiCore.attack.attack import Attack
-from wapitiCore.controller.wapiti import InvalidOptionValue
 from wapitiCore.net.response import Response
 from wapitiCore.wappalyzer.wappalyzer import Wappalyzer, ApplicationData, ApplicationDataException
 from wapitiCore.definitions.fingerprint import SoftwareNameDisclosureFinding
@@ -259,7 +259,6 @@ class ModuleWapp(Attack):
 
     async def attack(self, request: Request, response: Optional[Response] = None):
         self.finished = True
-        request_to_root = Request(request.url)
         categories_file_path = os.path.join(self.user_config_dir, self.WAPP_CATEGORIES)
         groups_file_path = os.path.join(self.user_config_dir, self.WAPP_GROUPS)
         technologies_file_path = os.path.join(self.user_config_dir, self.WAPP_TECHNOLOGIES)
@@ -279,7 +278,7 @@ class ModuleWapp(Attack):
             logging.error(exception)
             return
 
-        detected_applications, response = await self._detect_applications(request.url, application_data)
+        detected_applications, sent_request, response = await self._detect_applications(request.url, application_data)
 
         if detected_applications:
             log_blue("---")
@@ -300,7 +299,7 @@ class ModuleWapp(Attack):
             log_blue("")
             await self.add_info(
                 finding_class=SoftwareNameDisclosureFinding,
-                request=request_to_root,
+                request=sent_request,
                 info=json.dumps(detected_applications[application_name]),
                 response=response
             )
@@ -309,18 +308,21 @@ class ModuleWapp(Attack):
                 if "Web servers" in categories:
                     await self.add_info(
                         finding_class=WebServerVersionDisclosureFinding,
-                        request=request_to_root,
+                        request=sent_request,
                         info=json.dumps(detected_applications[application_name]),
                         response=response
                     )
                 else:
                     await self.add_info(
                         finding_class=SoftwareVersionDisclosureFinding,
-                        request=request_to_root,
+                        request=sent_request,
                         info=json.dumps(detected_applications[application_name]),
                         response=response
                     )
 
+                if application_name == "Apache HTTP Server":
+                    application_name = "apache"
+
                 if is_cve_supported_software(application_name):
                     if await self.download_latest_release_asset(application_name.lower(), self.user_config_dir):
                         async for severity, message in self.get_vulnerabilities(application_name.lower(), versions):
@@ -332,7 +334,7 @@ class ModuleWapp(Attack):
                                 finding_class=VulnerableSoftwareFinding,
                                 level=severity,
                                 info=message,
-                                request=request_to_root,
+                                request=sent_request,
                                 response=response
                             )
 
@@ -356,7 +358,7 @@ class ModuleWapp(Attack):
             self,
             url: str,
             application_data: ApplicationData
-    ) -> Tuple[Dict, Optional[Response]]:
+    ) -> Tuple[Dict, Request, Optional[Response]]:
         detected_applications = {}
         response = None
 
@@ -378,7 +380,7 @@ class ModuleWapp(Attack):
             wappalyzer = Wappalyzer(application_data, response, headless_results)
             detected_applications.update(wappalyzer.detect())
 
-        return detected_applications, response
+        return detected_applications, request, response
 
     async def _dump_url_content_to_file(self, url: str, file_path: str):
         request = Request(url)
