@@ -33,6 +33,7 @@ from wapitiCore.definitions.resource_consumption import ResourceConsumptionFindi
 from wapitiCore.definitions.internal_error import InternalErrorFinding
 from wapitiCore.model import PayloadInfo
 from wapitiCore.net import Request, Response
+from wapitiCore.net.web import http_repr
 from wapitiCore.parsers.ini_payload_parser import IniPayloadReader, replace_tags
 
 
@@ -89,8 +90,8 @@ class ModuleXxe(Attack):
         except RequestError:
             self.network_errors += 1
             return False
-        else:
-            return pattern in response.content
+
+        return pattern in response.content
 
     def flag_to_patterns(self, flags):
         try:
@@ -138,7 +139,7 @@ class ModuleXxe(Attack):
                 log_orange("---")
                 log_orange(Messages.MSG_TIMEOUT, page)
                 log_orange(Messages.MSG_EVIL_REQUEST)
-                log_orange(mutated_request.http_repr())
+                log_orange(http_repr(mutated_request))
                 log_orange("---")
 
                 if parameter.is_qs_injection:
@@ -147,7 +148,6 @@ class ModuleXxe(Attack):
                     anom_msg = Messages.MSG_PARAM_TIMEOUT.format(parameter.display_name)
 
                 await self.add_medium(
-                    request_id=request.path_id,
                     finding_class=ResourceConsumptionFinding,
                     request=mutated_request,
                     info=anom_msg,
@@ -167,7 +167,6 @@ class ModuleXxe(Attack):
                         vuln_message = f"{self.MSG_VULN} via injection in the parameter {parameter.display_name}"
 
                     await self.add_high(
-                        request_id=request.path_id,
                         finding_class=XxeFinding,
                         request=mutated_request,
                         info=vuln_message,
@@ -183,7 +182,7 @@ class ModuleXxe(Attack):
                         parameter.display_name
                     )
                     log_red(Messages.MSG_EVIL_REQUEST)
-                    log_red(mutated_request.http_repr())
+                    log_red(http_repr(mutated_request))
                     log_red("---")
 
                     # We reached maximum exploitation for this parameter, don't send more payloads
@@ -198,7 +197,6 @@ class ModuleXxe(Attack):
                         anom_msg = Messages.MSG_PARAM_500.format(parameter.display_name)
 
                     await self.add_high(
-                        request_id=request.path_id,
                         finding_class=InternalErrorFinding,
                         request=mutated_request,
                         info=anom_msg,
@@ -209,7 +207,7 @@ class ModuleXxe(Attack):
                     log_orange("---")
                     log_orange(Messages.MSG_500, page)
                     log_orange(Messages.MSG_EVIL_REQUEST)
-                    log_orange(mutated_request.http_repr())
+                    log_orange(http_repr(mutated_request))
                     log_orange("---")
 
     async def attack_body(self, original_request):
@@ -230,7 +228,6 @@ class ModuleXxe(Attack):
                 pattern = search_patterns(response.content, payload_info.rules)
                 if pattern and not await self.false_positive(original_request, pattern):
                     await self.add_high(
-                        request_id=original_request.path_id,
                         finding_class=XxeFinding,
                         request=mutated_request,
                         info="XXE vulnerability leading to file disclosure",
@@ -245,7 +242,7 @@ class ModuleXxe(Attack):
                         original_request.url
                     )
                     log_red(Messages.MSG_EVIL_REQUEST)
-                    log_red(mutated_request.http_repr())
+                    log_red(http_repr(mutated_request))
                     log_red("---")
                     self.vulnerables.add(original_request.path_id)
                     break
@@ -274,7 +271,6 @@ class ModuleXxe(Attack):
                 pattern = search_patterns(response.content, payload_info.rules)
                 if pattern and not await self.false_positive(original_request, pattern):
                     await self.add_high(
-                        request_id=original_request.path_id,
                         finding_class=XxeFinding,
                         request=mutated_request,
                         info="XXE vulnerability leading to file disclosure",
@@ -290,7 +286,7 @@ class ModuleXxe(Attack):
                         parameter.display_name
                     )
                     log_red(Messages.MSG_EVIL_REQUEST)
-                    log_red(mutated_request.http_repr())
+                    log_red(http_repr(mutated_request))
                     log_red("---")
                     vulnerable_parameter = True
                     self.vulnerables.add(original_request.path_id)
@@ -397,7 +393,6 @@ class ModuleXxe(Attack):
                         log_method = log_orange
 
                     await add_vuln_method(
-                        request_id=original_request.path_id,
                         finding_class=XxeFinding,
                         request=mutated_request,
                         info=vuln_message,
@@ -407,5 +402,5 @@ class ModuleXxe(Attack):
                     log_method("---")
                     log_method(vuln_message)
                     log_method(Messages.MSG_EVIL_REQUEST)
-                    log_method(mutated_request.http_repr())
+                    log_method(http_repr(mutated_request))
                     log_method("---")
